# Generated by Django 4.2.4 on 2023-09-07 03:29

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0014_schemalabel_schemadescription"),
    ]

    operations = [
        migrations.AlterField(
            model_name="formkitschema",
            name="label",
            field=models.CharField(
                blank=True,
                default=uuid.uuid4,
                help_text="Used as a human-readable label",
                max_length=1024,
                null=True,
                unique=True,
            ),
        ),
    ]
