# Generated by Django 4.2.4 on 2023-09-04 16:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0013_formkitschemanode_option_group"),
    ]

    operations = [
        migrations.CreateModel(
            name="SchemaLabel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=1024)),
                (
                    "lang",
                    models.CharField(
                        choices=[
                            ("en", "English"),
                            ("tet", "Tetum"),
                            ("pt", "Portugese"),
                        ],
                        default="en",
                        max_length=4,
                    ),
                ),
                (
                    "schema",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="formkit_ninja.formkitschema",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="SchemaDescription",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=1024)),
                (
                    "lang",
                    models.CharField(
                        choices=[
                            ("en", "English"),
                            ("tet", "Tetum"),
                            ("pt", "Portugese"),
                        ],
                        default="en",
                        max_length=4,
                    ),
                ),
                (
                    "schema",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="formkit_ninja.formkitschema",
                    ),
                ),
            ],
        ),
    ]
