"""
Don't make changes to this code directly
Instead, make changes to the template and re-generate this file
"""

from ninja import Router

from . import models, schema_out

router = Router(tags=["forms"])


{% for nodepath in nodepaths %}
{% with this = nodepath %}
{% include "api.jinja2" %}
{% endwith %}
{% endfor %}