# Generated by Django 4.2.4 on 2023-09-04 10:05

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0011_nodechildren_formkitschemanode_children"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="group",
        ),
        migrations.RemoveField(
            model_name="option",
            name="field",
        ),
        migrations.AlterField(
            model_name="formkitschema",
            name="label",
            field=models.CharField(
                blank=True,
                help_text="Used as a human-readable label",
                max_length=1024,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="nodechildren",
            name="child",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="formkit_ninja.formkitschemanode",
            ),
        ),
        migrations.AlterField(
            model_name="nodechildren",
            name="parent",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="parent",
                to="formkit_ninja.formkitschemanode",
            ),
        ),
        migrations.DeleteModel(
            name="Membership",
        ),
    ]
