# Generated by Django 4.2.6 on 2023-10-16 14:52

import pgtrigger.compiler
import pgtrigger.migrations
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0021_formkitschemanode_is_active"),
    ]

    operations = [
        pgtrigger.migrations.AddTrigger(
            model_name="formkitschemanode",
            trigger=pgtrigger.compiler.Trigger(
                name="soft_delete",
                sql=pgtrigger.compiler.UpsertTriggerSql(
                    func='UPDATE "formkit_ninja_formkitschemanode" SET is_active = False WHERE "id" = OLD."id"; RETURN NULL;',
                    hash="7e81f11bf057637a44641b9885a497624546c502",
                    operation="DELETE",
                    pgid="pgtrigger_soft_delete_69d8d",
                    table="formkit_ninja_formkitschemanode",
                    when="BEFORE",
                ),
            ),
        ),
    ]
