# Generated by Django 4.1.7 on 2023-05-08 01:01

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("formkit_ninja", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="formcomponents",
            name="label",
            field=models.CharField(
                help_text="Used as a human-readable label", max_length=1024
            ),
        ),
        migrations.AlterField(
            model_name="formkitschema",
            name="key",
            field=models.TextField(max_length=1024, unique=True),
        ),
    ]
